@@ -102,7 +102,7 @@ void KAssistantDialog::Private::init()
     q->connect(q, SIGNAL(user2Clicked()), q, SLOT(next()));
     q->connect(q, SIGNAL(user1Clicked()), q, SLOT(accept()));
 
-    q->connect(q, SIGNAL(currentPageChanged(KPageWidgetItem *, KPageWidgetItem *)), q, SLOT(_k_slotUpdateButtons()));
+    q->connect(q, SIGNAL(currentPageChanged(KPageWidgetItem*,KPageWidgetItem*)), q, SLOT(_k_slotUpdateButtons()));
 }
 
 
@@ -118,6 +118,8 @@ void KAssistantDialog::next()
     QModelIndex nextIndex=d->getNext(d->pageModel->index(currentPage()));
     if (nextIndex.isValid())
         setCurrentPage(d->pageModel->item(nextIndex));
+    else if (isValid(currentPage()))
+        accept();
 }
 
 void KAssistantDialog::setValid(KPageWidgetItem * page, bool enable)
