@@ -303,7 +303,7 @@ IconTextEditDialog::IconTextEditDialog(QWidget *parent)
     m_cbHidden = new QCheckBox(i18n("&Hide text when toolbar shows text alongside icons"), mainWidget);
     layout->addWidget(m_cbHidden, 1, 1);
 
-    connect(m_lineEdit, SIGNAL(textChanged(const QString &)), SLOT(slotTextChanged(const QString &)));
+    connect(m_lineEdit, SIGNAL(textChanged(QString)), SLOT(slotTextChanged(QString)));
 
     m_lineEdit->setFocus();
     setMainWidget(mainWidget);
@@ -342,15 +342,16 @@ class KEditToolBarWidgetPrivate
 public:
     /**
      *
-     * @param collection In a non-KParts application, this is the collection passed
+     * @param collection In the old-style constructor, this is the collection passed
      * to the KEditToolBar constructor.
-     * In a KParts application we let create a KXMLGUIClient create a dummy one,
+     * In the xmlguifactory-based constructor, we let KXMLGUIClient create a dummy one,
      * but it probably isn't used.
      */
     KEditToolBarWidgetPrivate(KEditToolBarWidget* widget,
                               const KComponentData &cData, KActionCollection* collection)
         : m_collection( collection ),
-          m_widget (widget),
+          m_widget(widget),
+          m_factory(NULL),
           m_loadedOnce( false )
     {
         m_componentData = cData;
@@ -390,8 +391,8 @@ public:
 
     void setupLayout();
 
-    void initNonKPart( const QString& file, bool global, const QString& defaultToolbar );
-    void initKPart( KXMLGUIFactory* factory, const QString& defaultToolbar );
+    void initOldStyle( const QString& file, bool global, const QString& defaultToolbar );
+    void initFromFactory( KXMLGUIFactory* factory, const QString& defaultToolbar );
     void loadToolBarCombo( const QString& defaultToolbar );
     void loadActions(const QDomElement& elem);
 
@@ -461,6 +462,7 @@ public:
     //QValueList<KAction*> m_actionList;
     KActionCollection* m_collection;
     KEditToolBarWidget* m_widget;
+    KXMLGUIFactory* m_factory;
     KComponentData m_componentData;
 
     QPixmap m_emptyIcon;
@@ -606,33 +608,23 @@ void KEditToolBarPrivate::_k_slotDefault()
 
     if ( m_factory )
     {
-        const QString localPrefix = KStandardDirs::locateLocal("data", "");
         foreach (KXMLGUIClient* client, m_factory->clients())
         {
-            QString file = client->xmlFile();
-
-            if (file.isNull()) // ##### should be isEmpty?
+            const QString file = client->localXMLFile();
+            if (file.isEmpty())
                 continue;
-
-            if (QDir::isRelativePath(file))
-            {
-                const KComponentData cData = client->componentData().isValid() ? client->componentData() : KGlobal::mainComponent();
-                file = KStandardDirs::locateLocal("data", cData.componentName() + '/' + file);
-            }
-            else
-            {
-                if (!file.startsWith(localPrefix))
-                    continue;
-            }
-
+            kDebug(240) << "Deleting local xml file" << file;
+            // << "for client" << client << typeid(*client).name();
             if ( QFile::exists( file ) )
                 if ( !QFile::remove( file ) )
-                    kWarning() << "Could not delete " << file;
+                    kWarning() << "Could not delete" << file;
         }
 
+        // Reload the xml files in all clients, now that the local files are gone
+        oldWidget->rebuildKXMLGUIClients();
+
         m_widget = new KEditToolBarWidget( q );
         m_widget->load( m_factory, m_defaultToolBar );
-        m_widget->rebuildKXMLGUIClients();
     }
     else
     {
@@ -658,6 +650,7 @@ void KEditToolBarPrivate::_k_slotDefault()
     q->connect(m_widget, SIGNAL(enableOk(bool)), SLOT(enableButtonApply(bool)));
 
     q->enableButtonApply(false);
+
     emit q->newToolBarConfig();
     emit q->newToolbarConfig(); // compat
 }
@@ -675,8 +668,12 @@ void KEditToolBarPrivate::_k_slotOk()
   }
   else
   {
-    emit q->newToolBarConfig();
-    emit q->newToolbarConfig(); // compat
+    // Do not emit the "newToolBarConfig" signal again here if the "Apply"
+    // button was already pressed and no further changes were made.
+    if (q->isButtonEnabled(KDialog::Apply)) {
+        emit q->newToolBarConfig();
+        emit q->newToolbarConfig(); // compat
+    }
     q->accept();
   }
 }
@@ -716,15 +713,15 @@ KEditToolBarWidget::~KEditToolBarWidget()
 
 void KEditToolBarWidget::load( const QString& file, bool global, const QString& defaultToolBar )
 {
-    d->initNonKPart( file, global, defaultToolBar );
+    d->initOldStyle( file, global, defaultToolBar );
 }
 
 void KEditToolBarWidget::load( KXMLGUIFactory* factory, const QString& defaultToolBar )
 {
-    d->initKPart( factory, defaultToolBar );
+    d->initFromFactory( factory, defaultToolBar );
 }
 
-void KEditToolBarWidgetPrivate::initNonKPart( const QString& resourceFile,
+void KEditToolBarWidgetPrivate::initOldStyle( const QString& resourceFile,
                                               bool global,
                                               const QString& defaultToolBar )
 {
@@ -738,7 +735,7 @@ void KEditToolBarWidgetPrivate::initNonKPart( const QString& resourceFile,
 
     // handle the merging
     if (global)
-        m_widget->setXMLFile(KStandardDirs::locate("config", "ui/ui_standards.rc"));
+        m_widget->loadStandardsXmlFile(); // ui_standards.rc
     const QString localXML = loadXMLFile( resourceFile );
     m_widget->setXML(localXML, global ? true /*merge*/ : false);
 
@@ -764,8 +761,8 @@ void KEditToolBarWidgetPrivate::initNonKPart( const QString& resourceFile,
     m_widget->setMinimumSize( m_widget->sizeHint() );
 }
 
-void KEditToolBarWidgetPrivate::initKPart( KXMLGUIFactory* factory,
-                                           const QString& defaultToolBar )
+void KEditToolBarWidgetPrivate::initFromFactory(KXMLGUIFactory* factory,
+                                                const QString& defaultToolBar)
 {
     //TODO: make sure we can call this multiple times?
     if ( m_loadedOnce ) {
@@ -774,10 +771,7 @@ void KEditToolBarWidgetPrivate::initKPart( KXMLGUIFactory* factory,
 
     m_loadedOnce = true;
 
-  // reusable vars
-  QDomElement elem;
-
-  m_widget->setFactory( factory );
+    m_factory = factory;
 
   // add all of the client data
   bool first = true;
@@ -790,6 +784,7 @@ void KEditToolBarWidgetPrivate::initKPart( KXMLGUIFactory* factory,
     if ( first ) {
       type = XmlData::Shell;
       first = false;
+      Q_ASSERT(!client->localXMLFile().isEmpty()); // where would we save changes??
     }
 
     XmlData data(type, client->localXMLFile(), client->actionCollection());
@@ -828,6 +823,16 @@ bool KEditToolBarWidget::save()
     if ( (*it).type() == XmlData::Merged )
       continue;
 
+    // Add noMerge="1" to all the menus since we are saving the merged data
+    QDomNodeList menuNodes = (*it).domDocument().elementsByTagName( "Menu" );
+    for (uint i = 0; i < menuNodes.length(); ++i)
+    {
+        QDomNode menuNode = menuNodes.item(i);
+        QDomElement menuElement = menuNode.toElement();
+        if (menuElement.isNull()) continue;
+        menuElement.setAttribute( "noMerge", "1" );
+    }
+
     kDebug() << (*it).domDocument().toString();
 
     kDebug(240) << "Saving " << (*it).xmlFile();
@@ -835,8 +840,8 @@ bool KEditToolBarWidget::save()
     KXMLGUIFactory::saveConfigFile((*it).domDocument(), (*it).xmlFile());
   }
 
-  if ( !factory() )
-    return true;
+    if (!d->m_factory)
+        return true;
 
   rebuildKXMLGUIClients();
 
@@ -845,23 +850,23 @@ bool KEditToolBarWidget::save()
 
 void KEditToolBarWidget::rebuildKXMLGUIClients()
 {
-  if ( !factory() )
-    return;
+    if (!d->m_factory)
+        return;
 
-  QList<KXMLGUIClient*> clients = factory()->clients();
-  //kDebug(240) << "factory: " << clients.count() << " clients";
+    const QList<KXMLGUIClient*> clients = d->m_factory->clients();
+    //kDebug(240) << "factory: " << clients.count() << " clients";
 
-  if (!clients.count())
-    return;
+    // remove the elements starting from the last going to the first
+    if (!clients.count())
+        return;
 
-  // remove the elements starting from the last going to the first
-  QListIterator<KXMLGUIClient*> clientIterator = clients;
-  clientIterator.toBack();
-  while ( clientIterator.hasPrevious() )
-  {
-    //kDebug(240) << "factory->removeClient " << client;
-    factory()->removeClient( clientIterator.previous() );
-  }
+    QListIterator<KXMLGUIClient*> clientIterator = clients;
+    clientIterator.toBack();
+    while (clientIterator.hasPrevious()) {
+        KXMLGUIClient* client = clientIterator.previous();
+        //kDebug(240) << "factory->removeClient " << client;
+        d->m_factory->removeClient(client);
+    }
 
   KXMLGUIClient *firstClient = clients.first();
 
@@ -878,18 +883,21 @@ void KEditToolBarWidget::rebuildKXMLGUIClients()
 
         // for the shell, merge in ui_standards.rc
         if ( client == firstClient ) // same assumption as in the ctor: first==shell
-            client->setXMLFile(KStandardDirs::locate("config", "ui/ui_standards.rc"));
+            client->loadStandardsXmlFile();
 
         // and this forces it to use the *new* XML file
         client->setXMLFile( file, client == firstClient /* merge if shell */ );
+
+        // [we can't use reloadXML, it doesn't load ui_standards.rc]
     }
   }
 
-  // Now we can add the clients to the factory
-  // We don't do it in the loop above because adding a part automatically
-  // adds its plugins, so we must make sure the plugins were updated first.
-  foreach (KXMLGUIClient* client, clients)
-    factory()->addClient( client );
+    // Now we can add the clients to the factory
+    // We don't do it in the loop above because adding a part automatically
+    // adds its plugins, so we must make sure the plugins were updated first.
+    foreach(KXMLGUIClient* client, clients) {
+        d->m_factory->addClient(client);
+    }
 }
 
 void KEditToolBarWidgetPrivate::setupLayout()
@@ -921,8 +929,8 @@ void KEditToolBarWidgetPrivate::setupLayout()
                    m_widget,       SLOT(slotInactiveSelectionChanged()));
   QObject::connect(m_inactiveList, SIGNAL(itemDoubleClicked(QListWidgetItem*)),
                    m_widget,       SLOT(slotInsertButton()));
-  QObject::connect(m_inactiveList, SIGNAL(dropped(ToolBarListWidget*, int, ToolBarItem*, bool)),
-                   m_widget,       SLOT(slotDropped(ToolBarListWidget*, int, ToolBarItem*, bool)));
+  QObject::connect(m_inactiveList, SIGNAL(dropped(ToolBarListWidget*,int,ToolBarItem*,bool)),
+                   m_widget,       SLOT(slotDropped(ToolBarListWidget*,int,ToolBarItem*,bool)));
 
   KListWidgetSearchLine *inactiveListSearchLine = new KListWidgetSearchLine(m_widget, m_inactiveList);
   inactiveListSearchLine->setClickMessage(i18n("Filter"));
@@ -940,8 +948,8 @@ void KEditToolBarWidgetPrivate::setupLayout()
                    m_widget,     SLOT(slotActiveSelectionChanged()));
   QObject::connect(m_activeList, SIGNAL(itemDoubleClicked(QListWidgetItem*)),
                    m_widget,     SLOT(slotRemoveButton()));
-  QObject::connect(m_activeList, SIGNAL(dropped(ToolBarListWidget*, int, ToolBarItem*, bool)),
-                   m_widget,     SLOT(slotDropped(ToolBarListWidget*, int, ToolBarItem*, bool)));
+  QObject::connect(m_activeList, SIGNAL(dropped(ToolBarListWidget*,int,ToolBarItem*,bool)),
+                   m_widget,     SLOT(slotDropped(ToolBarListWidget*,int,ToolBarItem*,bool)));
 
   KListWidgetSearchLine *activeListSearchLine = new KListWidgetSearchLine(m_widget, m_activeList);
   activeListSearchLine->setClickMessage(i18n("Filter"));
@@ -953,16 +961,16 @@ void KEditToolBarWidgetPrivate::setupLayout()
   m_hasKDialog = !kdialogExe.isEmpty();
   m_changeIcon->setEnabled(m_hasKDialog && m_activeList->currentItem());
 
-  QObject::connect( m_changeIcon, SIGNAL( clicked() ),
-                    m_widget, SLOT( slotChangeIcon() ) );
+  QObject::connect( m_changeIcon, SIGNAL(clicked()),
+                    m_widget, SLOT(slotChangeIcon()) );
 
   // "change icon text" button
   m_changeIconText = new KPushButton(i18n( "Change Te&xt..." ), m_widget);
   m_changeIconText->setIcon(KIcon("edit-rename"));
   m_changeIconText->setEnabled(m_activeList->currentItem() != 0);
 
-  QObject::connect( m_changeIconText, SIGNAL( clicked() ),
-                    m_widget, SLOT( slotChangeIconText() ) );
+  QObject::connect( m_changeIconText, SIGNAL(clicked()),
+                    m_widget, SLOT(slotChangeIconText()) );
 
   // The buttons in the middle
 
@@ -1289,8 +1297,6 @@ void KEditToolBarWidgetPrivate::slotInsertButton()
   // we're modified, so let this change
   emit m_widget->enableOk(true);
 
-  // TODO: #### this causes #97572.
-  // It would be better to just "delete item; loadActions( ... , ActiveListOnly );" or something.
   slotToolBarSelected( m_toolbarCombo->currentIndex() );
 
   selectActiveItem( internalName );
@@ -1545,8 +1551,8 @@ void KEditToolBarWidgetPrivate::slotChangeIcon()
   m_activeList->setEnabled( false ); // don't change the current item
   m_toolbarCombo->setEnabled( false ); // don't change the current toolbar
 
-  QObject::connect( m_kdialogProcess, SIGNAL( finished( int, QProcess::ExitStatus ) ),
-                    m_widget, SLOT( slotProcessExited() ) );
+  QObject::connect( m_kdialogProcess, SIGNAL(finished(int,QProcess::ExitStatus)),
+                    m_widget, SLOT(slotProcessExited()) );
 }
 
 void KEditToolBarWidgetPrivate::slotChangeIconText()
@@ -1673,7 +1679,7 @@ void KEditToolBar::showEvent( QShowEvent * event )
     if (!event->spontaneous()) {
         // The dialog has been shown, enable toolbar editing
         if ( d->m_factory ) {
-            // call the kpart version
+            // call the xmlgui-factory version
             d->m_widget->load( d->m_factory, d->m_defaultToolBar );
         } else {
             // call the action collection version
