@@ -97,6 +97,7 @@ KXMLGUIClient::~KXMLGUIClient()
   }
 
   if ( d->m_factory ) {
+      kWarning(240) << this << "deleted without having been removed from the factory first. This will leak standalone popupmenus and could lead to crashes.";
     d->m_factory->forgetClient(this);
   }
 
@@ -163,12 +164,18 @@ QString KXMLGUIClient::localXMLFile() const
   if ( !QDir::isRelativePath(d->m_xmlFile) )
       return QString(); // can't save anything here
 
+    if (d->m_xmlFile.isEmpty()) // setXMLFile not called at all, can't save. Use case: ToolBarHandler
+        return QString();
+
   return KStandardDirs::locateLocal( "data", componentData().componentName() + '/' + d->m_xmlFile );
 }
 
 
 void KXMLGUIClient::reloadXML()
 {
+    // TODO: this method can't be used for the KXmlGuiWindow, since it doesn't merge in ui_standards.rc!
+    //   -> KDE5: load ui_standards_rc in setXMLFile using a flag, and remember that flag?
+    //            and then KEditToolBar can use reloadXML.
     QString file( xmlFile() );
     if ( !file.isEmpty() )
         setXMLFile( file );
@@ -182,6 +189,17 @@ void KXMLGUIClient::setComponentData(const KComponentData &componentData)
     d->m_builder->setBuilderClient( this );
 }
 
+void KXMLGUIClient::loadStandardsXmlFile()
+{
+    const QString file = KStandardDirs::locate("config", "ui/ui_standards.rc", componentData());
+    if (file.isEmpty()) {
+        kWarning() << "ui/ui_standards.rc not found in" << componentData().dirs()->resourceDirs("config");
+    } else {
+        const QString doc = KXMLGUIFactory::readConfigFile( file );
+        setXML( doc );
+    }
+}
+
 void KXMLGUIClient::setXMLFile( const QString& _file, bool merge, bool setXMLDoc )
 {
   // store our xml file name
@@ -206,10 +224,10 @@ void KXMLGUIClient::setXMLFile( const QString& _file, bool merge, bool setXMLDoc
     kWarning() << "cannot find .rc file" << _file << "for component" << componentData().componentName();
   }
 
-  // make sure to merge the settings from any file specified by
-  // setLocalXMLFile()
-  if ( !d->m_localXMLFile.isEmpty() ) {
-    if ( !allFiles.contains( d->m_localXMLFile ) )
+  // make sure to merge the settings from any file specified by setLocalXMLFile()
+  if ( !d->m_localXMLFile.isEmpty() && !file.endsWith("ui_standards.rc") ) {
+    const bool exists = QDir::isRelativePath(d->m_localXMLFile) || QFile::exists(d->m_localXMLFile);
+    if (exists && !allFiles.contains(d->m_localXMLFile))
       allFiles.prepend( d->m_localXMLFile );
   }
 
@@ -217,17 +235,8 @@ void KXMLGUIClient::setXMLFile( const QString& _file, bool merge, bool setXMLDoc
   if ( !allFiles.isEmpty() )
     file = findMostRecentXMLFile(allFiles, doc);
 
-  if ( file.isEmpty() )
-  {
-    // this might or might not be an error.  for the time being,
-    // let's treat this as if it isn't a problem and the user just
-    // wants the global standards file
-    setXML( QString(), true );
-  }
-  else if ( !doc.isEmpty() )
-  {
-    setXML( doc, merge );
-  }
+  // Always call setXML, even on error, so that we don't keep all ui_standards.rc menus.
+  setXML( doc, merge );
 }
 
 void KXMLGUIClient::setLocalXMLFile( const QString &file )
