@@ -28,15 +28,18 @@
 #include <klocale.h>
 #include <solid/block.h>
 #include <solid/opticaldisc.h>
+#include <solid/opticaldrive.h>
 #include <solid/storageaccess.h>
 #include <solid/storagevolume.h>
 #include <solid/storagedrive.h>
+#include <solid/portablemediaplayer.h>
 
 
 KFilePlacesItem::KFilePlacesItem(KBookmarkManager *manager,
                                  const QString &address,
                                  const QString &udi)
-    : m_manager(manager), m_lister(0), m_folderIsEmpty(true), m_device(udi)
+    : m_manager(manager), m_lister(0), m_folderIsEmpty(true), m_isCdrom(false),
+      m_isAccessible(false), m_device(udi)
 {
     setBookmark(m_manager->findByAddress(address));
 
@@ -56,10 +59,14 @@ KFilePlacesItem::KFilePlacesItem(KBookmarkManager *manager,
         m_access = m_device.as<Solid::StorageAccess>();
         m_volume = m_device.as<Solid::StorageVolume>();
         m_disc = m_device.as<Solid::OpticalDisc>();
+        m_mtp = m_device.as<Solid::PortableMediaPlayer>();
         if (m_access) {
             connect(m_access, SIGNAL(accessibilityChanged(bool,QString)),
-                    this, SLOT(onAccessibilityChanged()));
+                    this, SLOT(onAccessibilityChanged(bool)));
+        onAccessibilityChanged(m_access->isAccessible());
         }
+        m_iconPath = m_device.icon();
+    m_emblems = m_device.emblems();
     }
 }
 
@@ -110,10 +117,12 @@ Solid::Device KFilePlacesItem::device() const
             m_access = m_device.as<Solid::StorageAccess>();
             m_volume = m_device.as<Solid::StorageVolume>();
             m_disc = m_device.as<Solid::OpticalDisc>();
+            m_mtp = m_device.as<Solid::PortableMediaPlayer>();
         } else {
             m_access = 0;
             m_volume = 0;
             m_disc = 0;
+            m_mtp = 0;
         }
     }
     return m_device;
@@ -171,19 +180,21 @@ QVariant KFilePlacesItem::deviceData(int role) const
         case Qt::DisplayRole:
             return d.description();
         case Qt::DecorationRole:
-            return KIcon(d.icon(), 0, d.emblems());
+            return KIcon(m_iconPath, 0, m_emblems);
         case KFilePlacesModel::UrlRole:
             if (m_access) {
                 return QUrl(KUrl(m_access->filePath()));
             } else if (m_disc && (m_disc->availableContent() & Solid::OpticalDisc::Audio)!=0) {
                 QString device = d.as<Solid::Block>()->device();
                 return QUrl(QString("audiocd:/?device=%1").arg(device));
+            } else if (m_mtp) {
+                return QUrl(QString("mtp:udi=%1").arg(d.udi()));
             } else {
                 return QVariant();
             }
         case KFilePlacesModel::SetupNeededRole:
             if (m_access) {
-                return !m_access->isAccessible();
+                return !m_isAccessible;
             } else {
                 return QVariant();
             }
@@ -203,16 +214,7 @@ QVariant KFilePlacesItem::deviceData(int role) const
             }
 
         case KFilePlacesModel::CapacityBarRecommendedRole:
-        {
-            bool accessible = m_access && m_access->isAccessible();
-            bool isCdrom =
-                ((m_device.is<Solid::StorageDrive>()
-                        && m_device.as<Solid::StorageDrive>()->driveType() == Solid::StorageDrive::CdromDrive)
-                || (m_device.parent().is<Solid::StorageDrive>()
-                        && m_device.parent().as<Solid::StorageDrive>()->driveType() == Solid::StorageDrive::CdromDrive));
-
-            return accessible && !isCdrom;
-        }
+        return m_isAccessible && !m_isCdrom;
 
         default:
             return QVariant();
@@ -291,8 +293,12 @@ QString KFilePlacesItem::generateNewId()
 //         + '/' + QString::number(qrand());
 }
 
-void KFilePlacesItem::onAccessibilityChanged()
+void KFilePlacesItem::onAccessibilityChanged(bool isAccessible)
 {
+    m_isAccessible = isAccessible;
+    m_isCdrom = m_device.is<Solid::OpticalDrive>() || m_device.parent().is<Solid::OpticalDrive>();
+    m_emblems = m_device.emblems();
+
     emit itemChanged(id());
 }
 
