@@ -27,12 +27,12 @@
 #define khtmlpart_p_h
 
 #include <kcursor.h>
-#include <klibloader.h>
 #include <kxmlguifactory.h>
 #include <kaction.h>
 #include <kparts/partmanager.h>
 #include <kparts/statusbarextension.h>
 #include <kparts/browserextension.h>
+#include <kparts/scriptableextension.h>
 
 #include <QtCore/QDate>
 #include <QtCore/QPointer>
@@ -48,12 +48,15 @@
 #include "khtml_events.h"
 #include "khtml_ext.h"
 #include "khtml_settings.h"
+#include "khtml_childframe_p.h"
+
 #include <kencodingdetector.h>
 #include "ecma/kjs_proxy.h"
 #include "xml/dom_nodeimpl.h"
 #include "editing/editing_p.h"
 #include "ui/findbar/khtmlfind_p.h"
 #include "ui/passwordbar/storepassbar.h"
+#include "ecma/kjs_scriptable.h"
 
 class KFind;
 class KFindDialog;
@@ -76,69 +79,16 @@ namespace KParts
 
 #include "khtml_wallet_p.h"
 
-namespace khtml
-{
-  class KHTML_EXPORT ChildFrame : public QObject
-  {
-      Q_OBJECT
-  public:
-      enum Type { Frame, IFrame, Object };
-
-      ChildFrame() : QObject (0) {
-          setObjectName( "khtml_child_frame" );
-          m_jscript = 0L;
-#ifndef DIRECT_LINKAGE_TO_ECMA
-          m_kjs_lib = 0;
-#endif
-          m_bCompleted = false; m_bPreloaded = false; m_type = Frame; m_bNotify = false;
-          m_bPendingRedirection = false;
-      }
-
-      ~ChildFrame() {
-          if (m_run) m_run->abort();
-          delete m_jscript;
-#ifndef DIRECT_LINKAGE_TO_ECMA
-          if ( m_kjs_lib)
-              m_kjs_lib->unload();
-#endif
-      }
-
-    QPointer<DOM::HTMLPartContainerElementImpl> m_partContainerElement;
-    QPointer<KParts::ReadOnlyPart> m_part;
-    QPointer<KParts::BrowserExtension> m_extension;
-    QPointer<KParts::LiveConnectExtension> m_liveconnect;
-    QString m_serviceName;
-    QString m_serviceType;
-    KJSProxy *m_jscript;
-#ifndef DIRECT_LINKAGE_TO_ECMA
-    KLibrary *m_kjs_lib;
-#endif
-    bool m_bCompleted;
-    QString m_name;
-    KParts::OpenUrlArguments m_args;
-    KParts::BrowserArguments m_browserArgs;
-    QPointer<KHTMLRun> m_run;
-    KUrl m_workingURL;
-    Type m_type;
-    QStringList m_params;
-    bool m_bPreloaded;
-    bool m_bNotify;
-    bool m_bPendingRedirection;
-  protected Q_SLOTS:
-    void liveConnectEvent(const unsigned long, const QString&, const KParts::LiveConnectExtension::ArgList&);
-  };
-
-}
-
-struct KHTMLFrameList : public QList<khtml::ChildFrame*>
-{
-    Iterator find( const QString &name ) KDE_NO_EXPORT;
+enum MimeType {
+    MimeHTML,
+    MimeSVG,
+    MimeXHTML,
+    MimeXML, // XML but not SVG or XHTML
+    MimeImage,
+    MimeText,
+    MimeOther
 };
 
-typedef KHTMLFrameList::ConstIterator ConstFrameIt;
-typedef KHTMLFrameList::Iterator FrameIt;
-
-
 
 class KHTMLPartPrivate
 {
@@ -193,6 +143,7 @@ public:
     m_bPluginsForce = false;
     m_bPluginsOverride = false;
     m_onlyLocalReferences = false;
+    m_forcePermitLocalImages = false;
     m_bDNSPrefetch = KHTMLPart::DNSPrefetchDisabled;
     m_bDNSPrefetchIsDefault = true;
     m_DNSPrefetchTimer = -1;
@@ -225,9 +176,10 @@ public:
             m_bPluginsOverride = part->d->m_bPluginsOverride;
             m_bDNSPrefetch = part->d->m_bDNSPrefetch;
             m_bDNSPrefetchIsDefault = part->d->m_bDNSPrefetchIsDefault;
+            m_onlyLocalReferences = part->d->m_onlyLocalReferences;
+            m_forcePermitLocalImages = part->d->m_forcePermitLocalImages;
             // Same for SSL settings
             m_ssl_in_use = part->d->m_ssl_in_use;
-            m_onlyLocalReferences = part->d->m_onlyLocalReferences;
             m_caretMode = part->d->m_caretMode;
             m_designMode = part->d->m_designMode;
             m_zoomFactor = part->d->m_zoomFactor;
@@ -255,6 +207,7 @@ public:
   ~KHTMLPartPrivate()
   {
     delete m_statusBarExtension;
+    delete m_scriptableExtension;
     delete m_extension;
     delete m_settings;
 #ifndef KHTML_NO_WALLET
@@ -277,6 +230,7 @@ public:
   KHTMLPartBrowserExtension *m_extension;
   KParts::StatusBarExtension *m_statusBarExtension;
   KHTMLPartBrowserHostExtension *m_hostExtension;
+  KJS::KHTMLPartScriptable *m_scriptableExtension;
   KUrlLabel* m_statusBarIconLabel;
   KUrlLabel* m_statusBarWalletLabel;
   KUrlLabel* m_statusBarUALabel;
@@ -349,6 +303,7 @@ public:
   bool m_bLoadEventEmitted;
   bool m_haveEncoding;
   bool m_onlyLocalReferences;
+  bool m_forcePermitLocalImages;
   bool m_redirectLockHistory;
 
   KUrl m_workingURL;
@@ -461,7 +416,7 @@ public:
   void setFlagRecursively(bool KHTMLPartPrivate::*flag, bool value);
 
   time_t m_userStyleSheetLastModified;
-  
+
   QSet<QString> m_lookedupHosts;
   static bool s_dnsInitialised;
 
@@ -469,6 +424,9 @@ public:
   KHTMLWalletQueue *m_wq;
 #endif
 
+  // Does determination of how we should handle the given type, as per HTML5 rules
+  MimeType classifyMimeType(const QString& mime);
+
   void clearRedirection();
 
   bool isLocalAnchorJump(const KUrl& url);
@@ -490,7 +448,22 @@ public:
       executeJavascriptURL(url);
   }
 
-  void propagateInitialDomainTo(KHTMLPart* kid);
+  void propagateInitialDomainAndBaseTo(KHTMLPart* kid);
+
+  void renameFrameForContainer(DOM::HTMLPartContainerElementImpl* cont,
+                               const QString& newName);
+                               
+  KHTMLPart* findFrameParent(KParts::ReadOnlyPart* callingPart, const QString& f, 
+                             khtml::ChildFrame **childFrame, bool checkForNavigation);
+                             
+  bool canNavigate(KParts::ReadOnlyPart* b);
+  KHTMLPart* top();
+
+  // Check whether the frame is fully loaded.
+  // The return value doesn't consider any pending redirections.
+  // If the return value is true, however, pendingRedirections will
+  // report if there are any
+  bool isFullyLoaded(bool* pendingRedirections) const;
 };
 
 #endif
