@@ -163,7 +163,6 @@ namespace khtml
 
 	bool isExpired() const;
 
-        virtual bool schedule() const { return false; }
 	virtual void finish();
 
         /**
@@ -229,7 +228,6 @@ namespace khtml
 	virtual void data( QBuffer &buffer, bool eof );
 	virtual void error( int err, const char *text );
 
-        virtual bool schedule() const { return true; }
         void setCharsetHint( const QString& charset ) { m_charsetHint = charset; }
         void setCharset( const QString& charset ) { m_charset = charset; }
         QString checkCharset(const QByteArray& buffer ) const;
@@ -261,8 +259,6 @@ namespace khtml
 	virtual void data( QBuffer &buffer, bool eof );
 	virtual void error( int err, const char *text );
 
-        virtual bool schedule() const { return false; }
-
 	void checkNotify();
 
         bool isLoaded() const { return !m_loading; }
@@ -315,10 +311,6 @@ namespace khtml
 #endif
 
         void setShowAnimations( KHTMLSettings::KAnimationAdvice );
-        void pauseAnimations();
-        void resumeAnimations();
-
-        virtual bool schedule() const { return true; }
 
 	virtual void finish();
 
@@ -347,21 +339,10 @@ namespace khtml
         QString m_suggestedTitle;
 #endif
         QPixmap* bg;
-	QPixmap* scaled;
+        QPixmap* scaled;
         QRgb bgColor;
         QSize bgSize;
 
-        ImageSource* imgSource;
-        const char* formatType;  // Is the name of the movie format type
-
-	int width;
-	int height;
-
-	// Is set if movie format type ( incremental/animation) was checked
-	bool typeChecked : 1;
-        bool isFullyTransparent : 1;
-        bool monochrome : 1;
-
         friend class Cache;
         friend class ::KHTMLPart;
     };
@@ -379,7 +360,6 @@ namespace khtml
 	virtual void ref(CachedObjectClient *consumer);
 	virtual void data( QBuffer &buffer, bool eof );
 	virtual void error( int err, const char *text );
-        virtual bool schedule() const { return false; }
 
 	void checkNotify();
 
@@ -402,7 +382,6 @@ namespace khtml
 	virtual void ref(CachedObjectClient *consumer);
 	virtual void data( QBuffer &buffer, bool eof );
 	virtual void error( int err, const char *text );
-        virtual bool schedule() const { return false; }
 
 	void checkNotify();
 
@@ -429,6 +408,7 @@ namespace khtml
         CachedScript *requestScript( const DOM::DOMString &url, const QString& charset);
         CachedSound *requestSound( const DOM::DOMString &url );
         CachedFont *requestFont( const DOM::DOMString &url );
+	bool willLoadMediaElement( const DOM::DOMString &url);
 
 	bool autoloadImages() const { return m_bautoloadImages; }
         KIO::CacheControl cachePolicy() const { return m_cachePolicy; }
@@ -442,8 +422,6 @@ namespace khtml
         void setAutoloadImages( bool );
         void setCachePolicy( KIO::CacheControl cachePolicy ) { m_cachePolicy = cachePolicy; }
         void setShowAnimations( KHTMLSettings::KAnimationAdvice );
-        void pauseAnimations();
-        void resumeAnimations();
         void insertCachedObject( CachedObject* o ) const;
         void removeCachedObject( CachedObject* o) const { m_docObjects.remove( o ); }
         void clearPreloads();
@@ -475,9 +453,10 @@ namespace khtml
     class Request
     {
     public:
-	Request(DocLoader* dl, CachedObject *_object, bool _incremental);
+	Request(DocLoader* dl, CachedObject *_object, bool _incremental, int priority);
 	~Request();
 	bool incremental;
+	int priority; // -10 to 10, smaller values mean higher priority
 	QBuffer m_buffer;
 	CachedObject *object;
         DocLoader* m_docLoader;
@@ -495,7 +474,7 @@ namespace khtml
 	Loader();
 	~Loader();
 
-	void load(DocLoader* dl, CachedObject *object, bool incremental = true, bool highPriority = false);
+	void load(DocLoader* dl, CachedObject *object, bool incremental = true, int priority = 0);
 
         int numRequests( DocLoader* dl ) const;
         void cancelRequests( DocLoader* dl );
@@ -512,13 +491,11 @@ namespace khtml
 	void slotFinished( KJob * );
 	void slotMimetype( KIO::Job *, const QString& s);
 	void slotData( KIO::Job *, const QByteArray & );
-	void servePendingRequests();
 
     protected:
-	QLinkedList<Request*> m_requestsPending;
-	Request* m_highPriorityRequestPending;
+        void scheduleRequest(Request* req);
+
 	QHash<KIO::Job*,Request*> m_requestsLoading;
-        QTimer m_timer;     
     };
 
         /**
