@@ -21,6 +21,7 @@
 #include "kfileitem.h"
 
 #include <config.h>
+#include <config-kio.h>
 
 #include <sys/time.h>
 #include <pwd.h>
@@ -36,12 +37,12 @@
 #include <QtCore/QFile>
 #include <QtCore/QMap>
 #include <QtGui/QApplication>
+#include <QtGui/QPalette>
 #include <QTextDocument>
 
 #include <kdebug.h>
 #include <kfilemetainfo.h>
 #include <kglobal.h>
-#include <kglobalsettings.h>
 #include <kiconloader.h>
 #include <klocale.h>
 #include <kmimetype.h>
@@ -54,6 +55,7 @@
 #include <knfsshare.h>
 #include <ksambashare.h>
 #endif
+#include <kfilesystemtype_p.h>
 
 class KFileItemPrivate : public QSharedData
 {
@@ -78,7 +80,8 @@ public:
           m_bMimeTypeKnown( false ),
           m_delayedMimeTypes( delayedMimeTypes ),
           m_useIconNameCache(false),
-          m_hidden( Auto )
+          m_hidden(Auto),
+          m_slow(SlowUnknown)
     {
         if (entry.count() != 0) {
             readUDSEntry( urlIsDirectory );
@@ -102,12 +105,14 @@ public:
      */
     void init();
 
+    QString localPath() const;
     KIO::filesize_t size() const;
     KDateTime time( KFileItem::FileTimes which ) const;
     void setTime(KFileItem::FileTimes which, long long time_t_val) const;
     bool cmp( const KFileItemPrivate & item ) const;
     QString user() const;
     QString group() const;
+    bool isSlow() const;
 
     /**
      * Extracts the data from the UDSEntry member and updates the KFileItem
@@ -186,10 +191,15 @@ public:
     // Auto: check leading dot.
     enum { Auto, Hidden, Shown } m_hidden:3;
 
+    // Slow? (nfs/smb/ssh)
+    mutable enum { SlowUnknown, Fast, Slow } m_slow:3;
+
     // For special case like link to dirs over FTP
     QString m_guessedMimeType;
     mutable QString m_access;
+#ifndef KDE_NO_DEPRECATED
     QMap<const void*, void*> m_extra; // DEPRECATED
+#endif
     mutable KFileMetaInfo m_metaInfo;
 
     enum { NumFlags = KFileItem::CreationTime + 1 };
@@ -501,6 +511,11 @@ KFileItem::~KFileItem()
 
 void KFileItem::refresh()
 {
+    if (!d) {
+        kWarning() << "null item";
+        return;
+    }
+
     d->m_fileMode = KFileItem::Unknown;
     d->m_permissions = KFileItem::Unknown;
     d->m_metaInfo = KFileMetaInfo();
@@ -517,6 +532,9 @@ void KFileItem::refresh()
 
 void KFileItem::refreshMimeType()
 {
+    if (!d)
+        return;
+
     d->m_pMimeType = 0;
     d->m_bMimeTypeKnown = false;
     d->m_iconName.clear();
@@ -524,12 +542,22 @@ void KFileItem::refreshMimeType()
 
 void KFileItem::setUrl( const KUrl &url )
 {
+    if (!d) {
+        kWarning() << "null item";
+        return;
+    }
+
     d->m_url = url;
     setName( url.fileName() );
 }
 
 void KFileItem::setName( const QString& name )
 {
+    if (!d) {
+        kWarning() << "null item";
+        return;
+    }
+
     d->m_strName = name;
     d->m_strText = KIO::decodeFileName( d->m_strName );
     if (d->m_entry.contains(KIO::UDSEntry::UDS_NAME))
@@ -539,6 +567,9 @@ void KFileItem::setName( const QString& name )
 
 QString KFileItem::linkDest() const
 {
+    if (!d)
+        return QString();
+
     // Extract it from the KIO::UDSEntry
     const QString linkStr = d->m_entry.stringValue( KIO::UDSEntry::UDS_LINK_DEST );
     if ( !linkStr.isEmpty() )
@@ -558,29 +589,46 @@ QString KFileItem::linkDest() const
     return QString();
 }
 
-QString KFileItem::localPath() const
+QString KFileItemPrivate::localPath() const
 {
-  if ( d->m_bIsLocalUrl ) {
-    return d->m_url.toLocalFile();
+  if (m_bIsLocalUrl) {
+    return m_url.toLocalFile();
   }
 
   // Extract the local path from the KIO::UDSEntry
-  return d->m_entry.stringValue( KIO::UDSEntry::UDS_LOCAL_PATH );
+  return m_entry.stringValue( KIO::UDSEntry::UDS_LOCAL_PATH );
+}
+
+QString KFileItem::localPath() const
+{
+    if (!d)
+        return QString();
+
+    return d->localPath();
 }
 
 KIO::filesize_t KFileItem::size() const
 {
+    if (!d)
+        return 0;
+
     return d->size();
 }
 
 bool KFileItem::hasExtendedACL() const
 {
+    if (!d)
+        return false;
+
     // Check if the field exists; its value doesn't matter
     return d->m_entry.contains(KIO::UDSEntry::UDS_EXTENDED_ACL);
 }
 
 KACL KFileItem::ACL() const
 {
+    if (!d)
+        return KACL();
+
     if ( hasExtendedACL() ) {
         // Extract it from the KIO::UDSEntry
         const QString fieldVal = d->m_entry.stringValue( KIO::UDSEntry::UDS_ACL_STRING );
@@ -593,6 +641,9 @@ KACL KFileItem::ACL() const
 
 KACL KFileItem::defaultACL() const
 {
+    if (!d)
+        return KACL();
+
     // Extract it from the KIO::UDSEntry
     const QString fieldVal = d->m_entry.stringValue( KIO::UDSEntry::UDS_DEFAULT_ACL_STRING );
     if ( !fieldVal.isEmpty() )
@@ -603,11 +654,18 @@ KACL KFileItem::defaultACL() const
 
 KDateTime KFileItem::time( FileTimes which ) const
 {
+    if (!d)
+        return KDateTime();
+
     return d->time(which);
 }
 
+#ifndef KDE_NO_DEPRECATED
 time_t KFileItem::time( unsigned int which ) const
 {
+    if (!d)
+        return 0;
+
     switch (which) {
     case KIO::UDSEntry::UDS_ACCESS_TIME:
         return d->time(AccessTime).toTime_t();
@@ -618,9 +676,13 @@ time_t KFileItem::time( unsigned int which ) const
         return d->time(ModificationTime).toTime_t();
     }
 }
+#endif
 
 QString KFileItem::user() const
 {
+    if (!d)
+        return QString();
+
     return d->user();
 }
 
@@ -649,6 +711,9 @@ QString KFileItemPrivate::user() const
 
 QString KFileItem::group() const
 {
+    if (!d)
+        return QString();
+
     return d->group();
 }
 
@@ -680,14 +745,42 @@ QString KFileItemPrivate::group() const
     return groupName;
 }
 
+bool KFileItemPrivate::isSlow() const
+{
+    if (m_slow == SlowUnknown) {
+        const QString path = localPath();
+        if (!path.isEmpty()) {
+            const KFileSystemType::Type fsType = KFileSystemType::fileSystemType(path);
+            m_slow = (fsType == KFileSystemType::Nfs || fsType == KFileSystemType::Smb) ? Slow : Fast;
+        } else {
+            m_slow = Slow;
+        }
+    }
+    return m_slow == Slow;
+}
+
+bool KFileItem::isSlow() const
+{
+    if (!d)
+        return false;
+
+    return d->isSlow();
+}
+
 QString KFileItem::mimetype() const
 {
+    if (!d)
+        return QString();
+
     KFileItem * that = const_cast<KFileItem *>(this);
     return that->determineMimeType()->name();
 }
 
 KMimeType::Ptr KFileItem::determineMimeType() const
 {
+    if (!d)
+        return KMimeType::Ptr();
+
     if ( !d->m_pMimeType || !d->m_bMimeTypeKnown )
     {
         bool isLocalUrl;
@@ -704,6 +797,9 @@ KMimeType::Ptr KFileItem::determineMimeType() const
 
 bool KFileItem::isMimeTypeKnown() const
 {
+    if (!d)
+        return false;
+
     // The mimetype isn't known if determineMimeType was never called (on-demand determination)
     // or if this fileitem has a guessed mimetype (e.g. ftp symlink) - in which case
     // it always remains "not fully determined"
@@ -712,6 +808,9 @@ bool KFileItem::isMimeTypeKnown() const
 
 QString KFileItem::mimeComment() const
 {
+    if (!d)
+        return QString();
+
     const QString displayType = d->m_entry.stringValue( KIO::UDSEntry::UDS_DISPLAY_TYPE );
     if (!displayType.isEmpty())
         return displayType;
@@ -724,14 +823,14 @@ QString KFileItem::mimeComment() const
     KMimeType::Ptr mime = mimeTypePtr();
     // This cannot move to kio_file (with UDS_DISPLAY_TYPE) because it needs
     // the mimetype to be determined, which is done here, and possibly delayed...
-    if (isLocalUrl && mime->is("application/x-desktop")) {
+    if (isLocalUrl && !d->isSlow() && mime->is("application/x-desktop")) {
         KDesktopFile cfg( url.toLocalFile() );
         QString comment = cfg.desktopGroup().readEntry( "Comment" );
         if (!comment.isEmpty())
             return comment;
     }
 
-    QString comment = mType->comment( url );
+    QString comment = d->isSlow() ? mType->comment() : mType->comment(url);
     //kDebug() << "finding comment for " << url.url() << " : " << d->m_pMimeType->name();
     if (!comment.isEmpty())
         return comment;
@@ -765,6 +864,9 @@ static QString iconFromDesktopFile(const QString& path)
 
 QString KFileItem::iconName() const
 {
+    if (!d)
+        return QString();
+
     if (d->m_useIconNameCache && !d->m_iconName.isEmpty()) {
         return d->m_iconName;
     }
@@ -778,8 +880,15 @@ QString KFileItem::iconName() const
     bool isLocalUrl;
     KUrl url = mostLocalUrl(isLocalUrl);
 
-    KMimeType::Ptr mime = mimeTypePtr();
-    if (isLocalUrl && mime->is("application/x-desktop")) {
+    KMimeType::Ptr mime;
+    // Use guessed mimetype for the icon
+    if (!d->m_guessedMimeType.isEmpty()) {
+        mime = KMimeType::mimeType( d->m_guessedMimeType );
+    } else {
+        mime = mimeTypePtr();
+    }
+
+    if (isLocalUrl && !isSlow() && mime->is("application/x-desktop")) {
         d->m_iconName = iconFromDesktopFile(url.toLocalFile());
         if (!d->m_iconName.isEmpty()) {
             d->m_useIconNameCache = d->m_bMimeTypeKnown;
@@ -790,7 +899,10 @@ QString KFileItem::iconName() const
     // KDE5: handle .directory files here too, and get rid of
     // KFolderMimeType and the url argument in KMimeType::iconName().
 
-    d->m_iconName = mime->iconName(url);
+    if (isSlow())
+        d->m_iconName = mime->iconName();
+    else
+        d->m_iconName = mime->iconName(url);
     d->m_useIconNameCache = d->m_bMimeTypeKnown;
     //kDebug() << "finding icon for" << url << ":" << d->m_iconName;
     return d->m_iconName;
@@ -823,6 +935,9 @@ static bool checkDesktopFile(const KFileItem& item, bool _determineMimeType)
 
 QStringList KFileItem::overlays() const
 {
+    if (!d)
+        return QStringList();
+
     QStringList names = d->m_entry.stringValue( KIO::UDSEntry::UDS_ICON_OVERLAY_NAMES ).split(',');
     if ( d->m_bLink ) {
         names.append("emblem-symbolic-link");
@@ -877,9 +992,20 @@ QStringList KFileItem::overlays() const
     return names;
 }
 
+QString KFileItem::comment() const
+{
+    if (!d)
+        return QString();
+
+    return d->m_entry.stringValue( KIO::UDSEntry::UDS_COMMENT );
+}
+
 // ## where is this used?
 QPixmap KFileItem::pixmap( int _size, int _state ) const
 {
+    if (!d)
+        return QPixmap();
+
     const QString iconName = d->m_entry.stringValue( KIO::UDSEntry::UDS_ICON_NAME );
     if ( !iconName.isEmpty() )
         return DesktopIcon(iconName, _size, _state);
@@ -903,8 +1029,8 @@ QPixmap KFileItem::pixmap( int _size, int _state ) const
     }
 
     KMimeType::Ptr mime;
-    // Use guessed mimetype if the main one hasn't been determined for sure
-    if ( !d->m_bMimeTypeKnown && !d->m_guessedMimeType.isEmpty() )
+    // Use guessed mimetype for the icon
+    if (!d->m_guessedMimeType.isEmpty())
         mime = KMimeType::mimeType( d->m_guessedMimeType );
     else
         mime = d->m_pMimeType;
@@ -932,6 +1058,9 @@ QPixmap KFileItem::pixmap( int _size, int _state ) const
 
 bool KFileItem::isReadable() const
 {
+    if (!d)
+        return false;
+
     /*
       struct passwd * user = getpwuid( geteuid() );
       bool isMyFile = (QString::fromLocal8Bit(user->pw_name) == d->m_user);
@@ -959,6 +1088,9 @@ bool KFileItem::isReadable() const
 
 bool KFileItem::isWritable() const
 {
+    if (!d)
+        return false;
+
     /*
       struct passwd * user = getpwuid( geteuid() );
       bool isMyFile = (QString::fromLocal8Bit(user->pw_name) == d->m_user);
@@ -982,6 +1114,9 @@ bool KFileItem::isWritable() const
 
 bool KFileItem::isHidden() const
 {
+    if (!d)
+        return false;
+
     // The kioslave can specify explicitly that a file is hidden or shown
     if ( d->m_hidden != KFileItemPrivate::Auto )
         return d->m_hidden == KFileItemPrivate::Hidden;
@@ -995,6 +1130,9 @@ bool KFileItem::isHidden() const
 
 bool KFileItem::isDir() const
 {
+    if (!d)
+        return false;
+
     if (d->m_fileMode == KFileItem::Unknown) {
         // Probably the file was deleted already, and KDirLister hasn't told the world yet.
         //kDebug() << d << url() << "can't say -> false";
@@ -1005,9 +1143,13 @@ bool KFileItem::isDir() const
 
 bool KFileItem::isFile() const
 {
+    if (!d)
+        return false;
+
     return !isDir();
 }
 
+#ifndef KDE_NO_DEPRECATED
 bool KFileItem::acceptsDrops() const
 {
     // A directory ?
@@ -1028,9 +1170,13 @@ bool KFileItem::acceptsDrops() const
 
     return false;
 }
+#endif
 
 QString KFileItem::getStatusBarInfo() const
 {
+    if (!d)
+        return QString();
+
     QString text = d->m_strText;
     const QString comment = mimeComment();
 
@@ -1057,8 +1203,12 @@ QString KFileItem::getStatusBarInfo() const
     return text;
 }
 
+#ifndef KDE_NO_DEPRECATED
 QString KFileItem::getToolTipText(int maxcount) const
 {
+    if (!d)
+        return QString();
+
     // we can return QString() if no tool tip should be shown
     QString tip;
     KFileMetaInfo info = metaInfo();
@@ -1112,11 +1262,6 @@ QString KFileItem::getToolTipText(int maxcount) const
             if ( item.isValid() )
             {
                 QString s = item.value().toString();
-                if ( ( item.properties().attributes() & PredicateProperties::SqueezeText )
-                     && s.length() > 50) {
-                    s.truncate(47);
-                    s.append("...");
-                }
                 if ( !s.isEmpty() )
                 {
                     count++;
@@ -1137,14 +1282,26 @@ QString KFileItem::getToolTipText(int maxcount) const
 
     return tip;
 }
+#endif
 
 void KFileItem::run( QWidget* parentWidget ) const
 {
+    if (!d) {
+        kWarning() << "null item";
+        return;
+    }
+
     (void) new KRun( targetUrl(), parentWidget, d->m_fileMode, d->m_bIsLocalUrl );
 }
 
 bool KFileItem::cmp( const KFileItem & item ) const
 {
+    if (!d && !item.d)
+        return true;
+
+    if (!d || !item.d)
+        return false;
+
     return d->cmp(*item.d);
 }
 
@@ -1159,9 +1316,13 @@ bool KFileItem::operator!=(const KFileItem& other) const
     return d != other.d;
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KFileItem::setUDSEntry( const KIO::UDSEntry& _entry, const KUrl& _url,
                              bool _delayedMimeTypes, bool _urlIsDirectory )
 {
+    if (!d)
+        return;
+
     d->m_entry = _entry;
     d->m_url = _url;
     d->m_strName.clear();
@@ -1184,32 +1345,51 @@ void KFileItem::setUDSEntry( const KIO::UDSEntry& _entry, const KUrl& _url,
     d->readUDSEntry( _urlIsDirectory );
     d->init();
 }
+#endif
 
 KFileItem::operator QVariant() const
 {
     return qVariantFromValue(*this);
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KFileItem::setExtraData( const void *key, void *value )
 {
+    if (!d)
+        return;
+
     if ( !key )
         return;
 
     d->m_extra.insert( key, value ); // replaces the value of key if already there
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 const void * KFileItem::extraData( const void *key ) const
 {
+    if (!d)
+        return 0;
+
     return d->m_extra.value( key, 0 );
 }
+#endif
 
+#ifndef KDE_NO_DEPRECATED
 void KFileItem::removeExtraData( const void *key )
 {
+    if (!d)
+        return;
+
     d->m_extra.remove( key );
 }
+#endif
 
 QString KFileItem::permissionsString() const
 {
+    if (!d)
+        return QString();
+
     if (d->m_access.isNull() && d->m_permissions != KFileItem::Unknown)
         d->m_access = d->parsePermissions( d->m_permissions );
 
@@ -1219,11 +1399,18 @@ QString KFileItem::permissionsString() const
 // check if we need to cache this
 QString KFileItem::timeString( FileTimes which ) const
 {
+    if (!d)
+        return QString();
+
     return KGlobal::locale()->formatDateTime( d->time(which) );
 }
 
+#ifndef KDE_NO_DEPRECATED
 QString KFileItem::timeString( unsigned int which ) const
 {
+    if (!d)
+        return QString();
+
     switch (which) {
     case KIO::UDSEntry::UDS_ACCESS_TIME:
         return timeString(AccessTime);
@@ -1234,14 +1421,21 @@ QString KFileItem::timeString( unsigned int which ) const
         return timeString(ModificationTime);
     }
 }
+#endif
 
 void KFileItem::setMetaInfo( const KFileMetaInfo & info ) const
 {
+    if (!d)
+        return;
+
     d->m_metaInfo = info;
 }
 
 KFileMetaInfo KFileItem::metaInfo(bool autoget, int what) const
 {
+    if (!d)
+        return KFileMetaInfo();
+
     if ((isRegularFile() || isDir()) && autoget && !d->m_metaInfo.isValid())
     {
         bool isLocalUrl;
@@ -1251,13 +1445,18 @@ KFileMetaInfo KFileItem::metaInfo(bool autoget, int what) const
     return d->m_metaInfo;
 }
 
+#ifndef KDE_NO_DEPRECATED
 void KFileItem::assign( const KFileItem & item )
 {
     *this = item;
 }
+#endif
 
 KUrl KFileItem::mostLocalUrl(bool &local) const
 {
+    if (!d)
+        return KUrl();
+
     QString local_path = localPath();
 
     if ( !local_path.isEmpty() )
@@ -1274,59 +1473,111 @@ KUrl KFileItem::mostLocalUrl(bool &local) const
     }
 }
 
+KUrl KFileItem::mostLocalUrl() const
+{
+    bool local = false;
+    return mostLocalUrl(local);
+}
+
 QDataStream & operator<< ( QDataStream & s, const KFileItem & a )
 {
-    // We don't need to save/restore anything that refresh() invalidates,
-    // since that means we can re-determine those by ourselves.
-    s << a.d->m_url;
-    s << a.d->m_strName;
-    s << a.d->m_strText;
+    if (a.d) {
+        // We don't need to save/restore anything that refresh() invalidates,
+        // since that means we can re-determine those by ourselves.
+        s << a.d->m_url;
+        s << a.d->m_strName;
+        s << a.d->m_strText;
+    } else {
+        s << KUrl();
+        s << QString();
+        s << QString();
+    }
+
     return s;
 }
 
 QDataStream & operator>> ( QDataStream & s, KFileItem & a )
 {
-    s >> a.d->m_url;
-    s >> a.d->m_strName;
-    s >> a.d->m_strText;
+    KUrl url;
+    QString strName, strText;
+
+    s >> url;
+    s >> strName;
+    s >> strText;
+
+    if (!a.d) {
+        kWarning() << "null item";
+        return s;
+    }
+
+    if (url.isEmpty()) {
+        a.d = 0;
+        return s;
+    }
+
+    a.d->m_url = url;
+    a.d->m_strName = strName;
+    a.d->m_strText = strText;
     a.d->m_bIsLocalUrl = a.d->m_url.isLocalFile();
     a.d->m_bMimeTypeKnown = false;
     a.refresh();
+
     return s;
 }
 
 KUrl KFileItem::url() const
 {
+    if (!d)
+        return KUrl();
+
     return d->m_url;
 }
 
 mode_t KFileItem::permissions() const
 {
+    if (!d)
+        return 0;
+
     return d->m_permissions;
 }
 
 mode_t KFileItem::mode() const
 {
+    if (!d)
+        return 0;
+
     return d->m_fileMode;
 }
 
 bool KFileItem::isLink() const
 {
+    if (!d)
+        return false;
+
     return d->m_bLink;
 }
 
 bool KFileItem::isLocalFile() const
 {
+    if (!d)
+        return false;
+
     return d->m_bIsLocalUrl;
 }
 
 QString KFileItem::text() const
 {
+    if (!d)
+        return QString();
+
     return d->m_strText;
 }
 
 QString KFileItem::name( bool lowerCase ) const
 {
+    if (!d)
+        return QString();
+
     if ( !lowerCase )
         return d->m_strName;
     else
@@ -1337,6 +1588,9 @@ QString KFileItem::name( bool lowerCase ) const
 
 KUrl KFileItem::targetUrl() const
 {
+    if (!d)
+        return KUrl();
+
     const QString targetUrlStr = d->m_entry.stringValue( KIO::UDSEntry::UDS_TARGET_URL );
     if (!targetUrlStr.isEmpty())
       return KUrl(targetUrlStr);
@@ -1346,6 +1600,10 @@ KUrl KFileItem::targetUrl() const
 
 KUrl KFileItem::nepomukUri() const
 {
+#ifndef KIO_NO_NEPOMUK
+    if (!d)
+        return KUrl();
+
     const QString nepomukUriStr = d->m_entry.stringValue( KIO::UDSEntry::UDS_NEPOMUK_URI );
     if(!nepomukUriStr.isEmpty()) {
         return KUrl(nepomukUriStr);
@@ -1356,6 +1614,9 @@ KUrl KFileItem::nepomukUri() const
     else {
         return KUrl();
     }
+#else
+    return KUrl();
+#endif
 }
 
 /*
@@ -1372,6 +1633,9 @@ KUrl KFileItem::nepomukUri() const
 
 KMimeType::Ptr KFileItem::mimeTypePtr() const
 {
+    if (!d)
+        return KMimeType::Ptr();
+
     if (!d->m_pMimeType) {
         // On-demand fast (but not always accurate) mimetype determination
         Q_ASSERT(!d->m_url.isEmpty());
@@ -1392,21 +1656,37 @@ KMimeType::Ptr KFileItem::mimeTypePtr() const
 
 KIO::UDSEntry KFileItem::entry() const
 {
+    if (!d)
+        return KIO::UDSEntry();
+
     return d->m_entry;
 }
 
 bool KFileItem::isMarked() const
 {
+    if (!d)
+        return false;
+
     return d->m_bMarked;
 }
 
 void KFileItem::mark()
 {
+    if (!d) {
+        kWarning() << "null item";
+        return;
+    }
+
     d->m_bMarked = true;
 }
 
 void KFileItem::unmark()
 {
+    if (!d) {
+        kWarning() << "null item";
+        return;
+    }
+
     d->m_bMarked = false;
 }
 
@@ -1481,6 +1761,9 @@ bool KFileItem::isDesktopFile() const
 
 bool KFileItem::isRegularFile() const
 {
+    if (!d)
+        return false;
+
     return S_ISREG(d->m_fileMode);
 }
 
