@@ -18,6 +18,9 @@
  *  Boston, MA 02110-1301, USA.
  */
 
+#undef QT_USE_FAST_OPERATOR_PLUS
+#undef QT_USE_FAST_CONCATENATION
+
 #include "krununittest.h"
 #include "krununittest.moc"
 #include <qtest_kde.h>
@@ -28,6 +31,7 @@ QTEST_KDEMAIN( KRunUnitTest, NoGUI )
 #include <kservice.h>
 #include <kstandarddirs.h>
 #include <kconfiggroup.h>
+#include <kprocess.h>
 #include "kiotesthelper.h" // createTestFile etc.
 
 void KRunUnitTest::initTestCase()
@@ -35,6 +39,11 @@ void KRunUnitTest::initTestCase()
     // testProcessDesktopExec works only if your terminal application is set to "x-term"
     KConfigGroup cg(KGlobal::config(), "General");
     cg.writeEntry("TerminalApplication", "x-term");
+
+    // Determine the full path of sh - this is needed to make testProcessDesktopExecNoFile()
+    // pass on systems where KStandardDirs::findExe("sh") is not "/bin/sh".
+    m_sh = KStandardDirs::findExe("sh");
+    if (m_sh.isEmpty()) m_sh = "/bin/sh";
 }
 
 void KRunUnitTest::testBinaryName_data()
@@ -92,15 +101,12 @@ static void checkPDE(const char* exec, const char* term, const char* sus,
 
 void KRunUnitTest::testProcessDesktopExec()
 {
-    QString sh = KStandardDirs::findExe("sh");
-    if (sh.isEmpty()) sh = "/bin/sh";
-
     KUrl::List l0;
     static const char
-        *execs[] = { "Exec=date -u", "Exec=echo $PWD" },
-        *terms[] = { "Terminal=false", "Terminal=true\nTerminalOptions=-T \"%f - %c\"" },
-          *sus[] = { "X-KDE-SubstituteUID=false", "X-KDE-SubstituteUID=true\nX-KDE-Username=sprallo" },
-        *rslts[] = {
+        * const execs[] = { "Exec=date -u", "Exec=echo $PWD" },
+        * const terms[] = { "Terminal=false", "Terminal=true\nTerminalOptions=-T \"%f - %c\"" },
+          * const sus[] = { "X-KDE-SubstituteUID=false", "X-KDE-SubstituteUID=true\nX-KDE-Username=sprallo" },
+        * const rslts[] = {
             "/bin/date -u", // 0
             "/bin/sh -c 'echo $PWD '", // 1
             "x-term -T ' - just_a_test' -e /bin/date -u", // 2
@@ -110,6 +116,15 @@ void KRunUnitTest::testProcessDesktopExec()
             "x-term -T ' - just_a_test' -e su sprallo -c '/bin/date -u'", // 6
             "x-term -T ' - just_a_test' -e su sprallo -c '/bin/sh -c '\\''echo $PWD '\\'''", // 7
         };
+
+    // Find out the full path of the shell which will be used to execute shell commands
+    KProcess process;
+    process.setShellCommand("");
+    const QString shellPath = process.program().at(0);
+
+    // Arch moved /bin/date to /usr/bin/date...
+    const QString datePath = KStandardDirs::findExe("date");
+
     for (int su = 0; su < 2; su++)
         for (int te = 0; te < 2; te++)
             for (int ex = 0; ex < 2; ex++) {
@@ -117,7 +132,9 @@ void KRunUnitTest::testProcessDesktopExec()
                 QString exe;
                 if (pt == 4 || pt == 5)
                     exe = KStandardDirs::findExe("kdesu");
-                const QString result = QString::fromLatin1(rslts[pt]).replace("/bin/sh", sh);
+                const QString result = QString::fromLatin1(rslts[pt])
+                    .replace("/bin/sh", shellPath)
+                    .replace("/bin/date", datePath);
                 checkPDE( execs[ex], terms[te], sus[su], l0, false, exe + result);
             }
 }
@@ -141,7 +158,8 @@ void KRunUnitTest::testProcessDesktopExecNoFile_data()
     if (kdeinit.isEmpty()) kdeinit = "kdeinit4";
 
     QString kioexec = KStandardDirs::findExe("kioexec");
-    if (kioexec.isEmpty()) kioexec = "kioexec";
+    if (kioexec.isEmpty())
+        QSKIP("kioexec not found, kdebase needed", SkipAll);
 
     QString kmailservice = KStandardDirs::findExe("kmailservice");
     if (kmailservice.isEmpty()) kmailservice = "kmailservice";
@@ -168,7 +186,7 @@ void KRunUnitTest::testProcessDesktopExecNoFile_data()
     QTest::newRow("%F l1 tempfile") << "kdeinit4 %F" << l1 << true << kioexec + " --tempfiles 'kdeinit4 %F' file:///tmp";
 
     QTest::newRow("sh -c kdeinit4 %F") << "sh -c \"kdeinit4 \"'\\\"'\"%F\"'\\\"'"
-                                   << l1 << false << "/bin/sh -c 'kdeinit4 \\\"/tmp\\\"'";
+                                   << l1 << false << m_sh + " -c 'kdeinit4 \\\"/tmp\\\"'";
 
     QTest::newRow("kmailservice %u l1") << "kmailservice %u" << l1 << false << kmailservice + " /tmp";
     QTest::newRow("kmailservice %u l4") << "kmailservice %u" << l4 << false << kmailservice + " http://login:password@www.kde.org";
@@ -224,7 +242,7 @@ void KRunUnitTest::testMimeTypeBrokenLink()
 {
     const QString dir = homeTmpDir() + "dir";
     createTestDirectory(dir);
-    KRunImpl* krun = new KRunImpl(dir + "/testlink", true);
+    KRunImpl* krun = new KRunImpl(KUrl(dir + "/testlink"), true);
     QSignalSpy spyError(krun, SIGNAL(error()));
     QTest::kWaitForSignal(krun, SIGNAL(finished()), 1000);
     QVERIFY(krun->mimeTypeFound().isEmpty());
