@@ -56,10 +56,9 @@ public:
           m_toolTip(parent),
           m_document(new QTextDocument(this))
     {
-        //d->text->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
-//        QTextOption op;
-//        op.setWrapMode(QTextOption::WordWrap);
-//        m_document->setDefaultTextOption(op);
+        QTextOption option = m_document->defaultTextOption();
+        option.setWrapMode(QTextOption::WordWrap);
+        m_document->setDefaultTextOption(option);
     }
 
     void setStyleSheet(const QString &css)
@@ -70,36 +69,33 @@ public:
     void setContent(const ToolTipContent &data)
     {
         QString html;
-        if (!data.mainText().isEmpty()) {
-            html.append("<b>" + data.mainText() + "</b>");
-
-            if (!data.subText().isEmpty()) {
-                html.append("<br>");
+        QString mainText = data.mainText();
+        if (!mainText.isEmpty()) {
+            if (mainText.size() < 50) {
+                // don't let short texts wrap on us!
+                mainText = mainText.replace(" ", "&nbsp;");
             }
+            html.append("<div align=\"center\"><b>" + mainText + "</b></div>");
         }
         html.append(data.subText());
 
         m_anchor.clear();
         m_document->clear();
         data.registerResources(m_document);
-        m_document->setHtml("<p>" + html + "</p>");
-        m_document->adjustSize();
-
-        m_haloRects.clear();
-        QTextLayout *layout = m_document->begin().layout();
-        //layout->setPosition(QPointF(textRect.x(), textBoundingRect->y()));
-        QTextLine line;
-        for (int i = 0; i < layout->lineCount(); ++i) {
-            line = layout->lineAt(i);
-            m_haloRects.append(line.naturalTextRect().translated(layout->position().toPoint()).toRect().translated(m_margin, m_margin));
+        if (!html.isEmpty()) {
+            m_document->setHtml("<p>" + html + "</p>");
+        } else {
+            m_document->clear();
         }
+        m_document->adjustSize();
 
         update();
     }
 
     QSize minimumSizeHint() const
     {
-        return m_document->size().toSize() + QSize(m_margin, m_margin)*2;
+        const int margin = 6;
+        return m_document->size().toSize() + QSize(margin, margin)*2;
     }
 
     QSize maximumSizeHint() const
@@ -110,14 +106,6 @@ public:
     void paintEvent(QPaintEvent *event)
     {
         QPainter p(this);
-
-        if (Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor).value() < 128) {
-            foreach (const QRectF &rect, m_haloRects) {
-                Plasma::PaintUtils::drawHalo(&p, rect);
-            }
-
-            p.translate(m_margin, m_margin);
-        }
         m_document->drawContents(&p, event->rect());
     }
 
@@ -147,8 +135,6 @@ private:
     ToolTip *m_toolTip;
     QTextDocument *m_document;
     QString m_anchor;
-    QList<QRectF> m_haloRects;
-    static const int m_margin = 6;
 };
 
 class ToolTipPrivate
@@ -181,8 +167,8 @@ ToolTip::ToolTip(QWidget *parent)
     d->preview = new WindowPreview(this);
     d->text = new TipTextWidget(this);
     d->imageLabel = new QLabel(this);
-    d->imageLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
 
+    d->imageLabel->setAlignment(Qt::AlignTop | Qt::AlignLeft);
     d->animation = new QPropertyAnimation(this, "pos", this);
     d->animation->setEasingCurve(QEasingCurve::InOutQuad);
     d->animation->setDuration(250);
@@ -200,7 +186,7 @@ ToolTip::ToolTip(QWidget *parent)
 
     QHBoxLayout *iconTextHBoxLayout = new QHBoxLayout;
     iconTextHBoxLayout->addWidget(d->imageLabel);
-    iconTextHBoxLayout->setAlignment(d->imageLabel, Qt::AlignCenter);
+    iconTextHBoxLayout->setAlignment(d->imageLabel, Qt::AlignTop | Qt::AlignHCenter);
     iconTextHBoxLayout->addWidget(d->text);
     iconTextHBoxLayout->setAlignment(d->text, Qt::AlignLeft | Qt::AlignVCenter);
     iconTextHBoxLayout->setStretchFactor(d->text, 1);
@@ -228,9 +214,13 @@ void ToolTip::showEvent(QShowEvent *e)
 void ToolTip::hideEvent(QHideEvent *e)
 {
     QWidget::hideEvent(e);
-    if (d->source) {
-        QMetaObject::invokeMethod(d->source.data(), "toolTipHidden");
+    d->animation->stop();
+
+    QObject *source = d->source.data();
+    if (source && source->metaObject()->indexOfMethod("toolTipHidden()") != -1) {
+        QMetaObject::invokeMethod(source, "toolTipHidden");
     }
+
     WindowEffects::highlightWindows(winId(), QList<WId>());
 }
 
@@ -298,20 +288,18 @@ void ToolTip::setContent(QObject *tipper, const ToolTipContent &data)
 {
     //reset our size
     d->text->setContent(data);
-    d->imageLabel->setPixmap(data.image());
+    if (data.image().isNull()) {
+        d->imageLabel->hide();
+    } else {
+        d->imageLabel->show();
+        d->imageLabel->setPixmap(data.image());
+    }
 
-    if (data.highlightWindows() && (data.windowsToPreview().size() > 1 || data.windowToPreview() != 0)) {
+    if (data.highlightWindows() && !data.windowsToPreview().isEmpty()) {
         WindowEffects::highlightWindows(winId(), QList<WId>() << winId() << data.windowsToPreview());
     }
 
-    if (data.windowsToPreview().size() > 1) {
-        d->preview->setWindowIds(data.windowsToPreview());
-    } else {
-        QList<WId>ids;
-        ids.append(data.windowToPreview());
-        d->preview->setWindowIds(ids);
-    }
-
+    d->preview->setWindowIds(data.windowsToPreview());
     d->preview->setHighlightWindows(data.highlightWindows());
 
     d->autohide = data.autohide();
@@ -326,12 +314,8 @@ void ToolTip::setContent(QObject *tipper, const ToolTipContent &data)
 
 void ToolTip::prepareShowing()
 {
-    if (!d->preview->isEmpty()) {
-        // show/hide the preview area
-        d->preview->show();
-    } else {
-        d->preview->hide();
-    }
+    // show/hide the preview area
+    d->preview->setVisible(!d->preview->isEmpty());
 
     layout()->activate();
     d->preview->setInfo();
@@ -347,9 +331,7 @@ void ToolTip::moveTo(const QPoint &to)
         return;
     }
 
-    if (d->animation->state() == QAbstractAnimation::Running) {
-        d->animation->stop();
-    }
+    d->animation->stop();
     d->animation->setEndValue(to);
     d->animation->start();
 }
@@ -374,10 +356,9 @@ void ToolTip::resizeEvent(QResizeEvent *e)
 void ToolTip::paintEvent(QPaintEvent *e)
 {
     QPainter painter(this);
-    painter.setRenderHint(QPainter::Antialiasing);
     painter.setClipRect(e->rect());
     painter.setCompositionMode(QPainter::CompositionMode_Source);
-    painter.fillRect(rect(), Qt::transparent);
+    painter.fillRect(e->rect(), Qt::transparent);
 
     d->background->paintFrame(&painter);
 }
